var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
import { Channel } from './Channel';
import { defaultOptions } from './service/WebChannel';
import { wcs, WebGroup } from './WebChannelFacade';
import { WebSocketBuilder } from './WebSocketBuilder';
var urlLib = require('url');
var uws = require('uws');
var BotServer = /** @class */ (function () {
    function BotServer(_a) {
        var _b = _a.url, url = _b === void 0 ? '' : _b, _c = _a.perMessageDeflate, perMessageDeflate = _c === void 0 ? false : _c, server = _a.server, _d = _a.webGroupOptions, webGroupOptions = _d === void 0 ? {
            topology: defaultOptions.topology,
            signalingServer: defaultOptions.signalingServer,
            rtcConfiguration: defaultOptions.rtcConfiguration,
            autoRejoin: false,
        } : _d;
        // public
        this.wcOptions = Object.assign({}, defaultOptions, { autoRejoin: false }, webGroupOptions);
        this.server = server;
        this.listenUrl = url;
        this.perMessageDeflate = perMessageDeflate;
        // private
        this.webGroups = new Set();
        this.onWebGroup = function none() { };
        this.onError = function none() { };
        // initialize server
        this.init();
    }
    Object.defineProperty(BotServer.prototype, "url", {
        get: function () {
            if (this.listenUrl !== '') {
                return this.listenUrl;
            }
            else {
                var info = this.server.address();
                return "ws://" + info.address + ":" + info.port;
            }
        },
        enumerable: true,
        configurable: true
    });
    BotServer.prototype.getWebGroup = function (id) {
        try {
            for (var _a = __values(this.webGroups), _b = _a.next(); !_b.done; _b = _a.next()) {
                var wg = _b.value;
                if (id === wg.id) {
                    return wg;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return undefined;
        var e_1, _c;
    };
    BotServer.prototype.init = function () {
        var _this = this;
        this.webSocketServer = new uws.Server({
            perMessageDeflate: this.perMessageDeflate,
            verifyClient: function (info) { return _this.validateConnection(info); },
            server: this.server,
        });
        var serverListening = this.server || this.webSocketServer;
        serverListening.on('listening', function () { return WebSocketBuilder.listen().next(_this.url); });
        this.webSocketServer.on('error', function (err) {
            WebSocketBuilder.listen().next('');
            _this.onError(err);
        });
        this.webSocketServer.on('connection', function (ws) {
            var _a = urlLib.parse(ws.upgradeReq.url, true), pathname = _a.pathname, query = _a.query;
            var wcId = Number(query.wcId);
            var wg = _this.getWebGroup(wcId);
            var senderId = Number(query.senderId);
            switch (pathname) {
                case '/invite': {
                    if (wg && wg.members.length === 1) {
                        _this.webGroups.delete(wg);
                    }
                    // FIXME: it is possible to create multiple WebChannels with the same ID
                    wg = new WebGroup(_this.wcOptions);
                    var wc = wcs.get(wg);
                    wc.id = wcId;
                    _this.webGroups.add(wg);
                    _this.onWebGroup(wg);
                    new Channel(wc, ws, { id: senderId }); // tslint:disable-line
                    break;
                }
                case '/internalChannel': {
                    if (wg !== undefined) {
                        WebSocketBuilder.newIncomingSocket(wcs.get(wg), ws, senderId);
                    }
                    else {
                        console.error('Cannot find WebChannel for a new internal channel');
                    }
                    break;
                }
            }
        });
    };
    BotServer.prototype.validateConnection = function (info) {
        var _a = urlLib.parse(info.req.url, true), pathname = _a.pathname, query = _a.query;
        var wcId = query.wcId ? Number(query.wcId) : undefined;
        switch (pathname) {
            case '/invite':
                if (wcId) {
                    var wg = this.getWebGroup(wcId);
                    return (wg === undefined || wg.members.length === 1) && query.senderId;
                }
                return false;
            case '/internalChannel':
                return query.senderId !== undefined && wcId !== undefined && this.getWebGroup(wcId) !== undefined;
            default:
                return false;
        }
    };
    return BotServer;
}());
export { BotServer };
