import { Observable } from 'rxjs/Observable';
import { Channel } from './Channel';
import { signaling as sigProto } from './proto';
import { WebChannel } from './service/WebChannel';
export interface IWebRTCStream {
    message: Observable<sigProto.Content>;
    send: (msg: sigProto.IContent) => void;
}
export declare enum SignalingState {
    CONNECTING = 0,
    CONNECTED = 1,
    STABLE = 2,
    CLOSING = 3,
    CLOSED = 4,
}
/**
 * This class represents a door of the `WebChannel` for the current peer. If the door
 * is open, then clients can join the `WebChannel` through this peer. There are as
 * many doors as peers in the `WebChannel` and each of them can be closed or opened.
 */
export declare class Signaling {
    url: string;
    state: SignalingState;
    private wc;
    private stateSubject;
    private channelSubject;
    private ws;
    private wsObservable;
    private completeWs;
    private heartbeatInterval;
    private missedHeartbeat;
    constructor(wc: WebChannel, url: string);
    readonly onState: Observable<SignalingState>;
    readonly onChannel: Observable<Channel>;
    /**
     * Notify Signaling server that you had joined the network and ready
     * to join new peers to the network.
     */
    open(): void;
    join(key: string): void;
    /**
     * Close the `WebSocket` with Signaling server.
     */
    close(): void;
    private connectOverSignaling();
    private setState(state);
    private startHeartbeat();
    private createObservable(ws);
    private send(msg);
    private heartbeat();
    private getWebRTCStream();
    private getFullURL(params);
}
