import { BehaviorSubject } from 'rxjs/BehaviorSubject';
import { Subject } from 'rxjs/Subject';
import { Channel } from './Channel';
import { isURL } from './misc/Util';
export var CONNECT_TIMEOUT = 6000;
var listenSubject = new BehaviorSubject('');
/**
 * Service class responsible to establish connections between peers via
 * `WebSocket`.
 */
var WebSocketBuilder = /** @class */ (function () {
    function WebSocketBuilder(wc) {
        this.wc = wc;
        this.channelsSubject = new Subject();
    }
    WebSocketBuilder.listen = function () {
        return listenSubject;
    };
    WebSocketBuilder.newIncomingSocket = function (wc, ws, senderId) {
        wc.webSocketBuilder.channelsSubject.next(new Channel(wc, ws, { id: senderId }));
    };
    Object.defineProperty(WebSocketBuilder.prototype, "onChannel", {
        get: function () {
            return this.channelsSubject.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Establish `WebSocket` with a server if `id` is not specified,
     * otherwise return an opened `Channel` with a peer identified by the
     * specified `id`.
     *
     * @param url Server URL
     * @param id  Peer id
     */
    WebSocketBuilder.prototype.connect = function (url, id) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            try {
                if (isURL(url) && url.search(/^wss?/) !== -1) {
                    var fullUrl = id !== undefined ? url + "/internalChannel?wcId=" + _this.wc.id + "&senderId=" + _this.wc.myId : url;
                    var ws_1 = new global.WebSocket(fullUrl);
                    var timeout_1 = setTimeout(function () {
                        if (ws_1.readyState !== ws_1.OPEN) {
                            ws_1.close();
                            reject(new Error("WebSocket " + CONNECT_TIMEOUT + "ms connection timeout with '" + url + "'"));
                        }
                    }, CONNECT_TIMEOUT);
                    ws_1.onopen = function () {
                        clearTimeout(timeout_1);
                        if (id === undefined) {
                            resolve(ws_1);
                        }
                        else {
                            resolve(new Channel(_this.wc, ws_1, { id: id }));
                        }
                    };
                    ws_1.onerror = function (err) { return reject(err); };
                    ws_1.onclose = function (closeEvt) { return reject(new Error("WebSocket connection to '" + url + "' failed with code " + closeEvt.code + ": " + closeEvt.reason)); };
                }
                else {
                    reject(new Error(url + " is not a valid URL"));
                }
            }
            catch (err) {
                reject(err);
            }
        });
    };
    return WebSocketBuilder;
}());
export { WebSocketBuilder };
