import { LogLevel as Logs } from './misc/Util';
export { WebGroup, WebGroupOptions, WebGroupState, DataType } from './WebChannelFacade';
/**
 * Set log level for debugging utility. By default all logs are disabled.
 * @param {...LogLevel} levels
 */
export declare function setLogLevel(...levels: Logs[]): void;
/**
 * The state enum of the signaling server for WebRTC.
 */
export declare class SignalingState {
    /**
     * Equals to `0`, the connection is not yet open. Value
     * @type {number}
     */
    static readonly CONNECTING: number;
    /**
     * Equals to `1`, `RTCDataChannel` has been established with one of the group member.
     * From now the signaling is no longer needed, because the joining process
     * will continue with a help of this member.
     * @type {number}
     */
    static readonly CONNECTED: number;
    /**
     * Equals to `2`, you has successfully joined a web group and ready to help join others.
     * @type {number}
     */
    static readonly STABLE: number;
    /**
     * Equals to `3`, the connection is in process of closing.
     * @type {number}
     */
    static readonly CLOSING: number;
    /**
     * Equals to `4`, the connection is closed.
     * @type {number}
     */
    static readonly CLOSED: number;
}
/**
 * The topology enum.
 */
export declare class Topology {
    /**
     * Full mesh topology identifier.
     * @type {number}
     */
    static readonly FULL_MESH: number;
}
/**
 * The log level enum.
 */
export declare class LogLevel {
    /**
     * Equals to `1`, allows logs for debug.
     * @type {number}
     */
    static readonly DEBUG: number;
    /**
     * Equals to `2`, logs for WebGroup module.
     * @type {number}
     */
    static readonly WEB_GROUP: number;
    /**
     * Equals to `3`, logs for WebRTCBuilder module.
     * @type {number}
     */
    static readonly WEBRTC: number;
    /**
     * Equals to `4`, logs for Channel module.
     * @type {number}
     */
    static readonly CHANNEL: number;
    /**
     * Equals to `5`, logs for Topology module.
     * @type {number}
     */
    static readonly TOPOLOGY: number;
    /**
     * Equals to `6`, logs for Signaling module.
     * @type {number}
     */
    static readonly SIGNALING: number;
    /**
     * Equals to `7`, logs for ChannelBuilder module.
     * @type {number}
     */
    static readonly CHANNEL_BUILDER: number;
}
