import { LogLevel as Logs, setLogLevel as setLogs } from './misc/Util';
import { TopologyEnum } from './service/topology/Topology';
import { SignalingState as SigState } from './Signaling';
export { WebGroup, WebGroupState } from './WebChannelFacade';
/**
 * Set log level for debugging utility. By default all logs are disabled.
 * @param {...LogLevel} levels
 */
export function setLogLevel() {
    var levels = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        levels[_i] = arguments[_i];
    }
    setLogs(levels);
}
/**
 * The state enum of the signaling server for WebRTC.
 */
var SignalingState = /** @class */ (function () {
    function SignalingState() {
    }
    Object.defineProperty(SignalingState, "CONNECTING", {
        /**
         * Equals to `0`, the connection is not yet open. Value
         * @type {number}
         */
        get: function () { return SigState.CONNECTING; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SignalingState, SignalingState.CONNECTING, {
        /**
         * Equals to `'CONNECTING'`.
         * @type {string}
         */
        get: function () { return SigState[SigState.CONNECTING]; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SignalingState, "CONNECTED", {
        /**
         * Equals to `1`, `RTCDataChannel` has been established with one of the group member.
         * From now the signaling is no longer needed, because the joining process
         * will continue with a help of this member.
         * @type {number}
         */
        get: function () { return SigState.CONNECTED; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SignalingState, SignalingState.CONNECTED, {
        /**
         * Equals to `'CONNECTED'`.
         * @type {string}
         */
        get: function () {
            return SigState[SigState.CONNECTED];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SignalingState, "STABLE", {
        /**
         * Equals to `2`, you has successfully joined a web group and ready to help join others.
         * @type {number}
         */
        get: function () { return SigState.STABLE; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SignalingState, SignalingState.STABLE, {
        /**
         * Equals to `'STABLE'`.
         * @type {string}
         */
        get: function () { return SigState[SigState.STABLE]; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SignalingState, "CLOSING", {
        /**
         * Equals to `3`, the connection is in process of closing.
         * @type {number}
         */
        get: function () { return SigState.CLOSING; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SignalingState, SignalingState.CLOSING, {
        /**
         * Equals to `'CLOSING'`.
         * @type {string}
         */
        get: function () { return SigState[SigState.CLOSING]; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SignalingState, "CLOSED", {
        /**
         * Equals to `4`, the connection is closed.
         * @type {number}
         */
        get: function () { return SigState.CLOSED; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SignalingState, SignalingState.CLOSED, {
        /**
         * Equals to `'CLOSED'`.
         * @type {string}
         */
        get: function () { return SigState[SigState.CLOSED]; },
        enumerable: true,
        configurable: true
    });
    return SignalingState;
}());
export { SignalingState };
/**
 * The topology enum.
 */
var Topology = /** @class */ (function () {
    function Topology() {
    }
    Object.defineProperty(Topology, "FULL_MESH", {
        /**
         * Full mesh topology identifier.
         * @type {number}
         */
        get: function () { return TopologyEnum.FULL_MESH; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Topology, Topology.FULL_MESH, {
        /**
         * Equals to `'FULL_MESH'`.
         * @type {string}
         */
        get: function () { return TopologyEnum[TopologyEnum.FULL_MESH]; },
        enumerable: true,
        configurable: true
    });
    return Topology;
}());
export { Topology };
/**
 * The log level enum.
 */
var LogLevel = /** @class */ (function () {
    function LogLevel() {
    }
    Object.defineProperty(LogLevel, "DEBUG", {
        /**
         * Equals to `1`, allows logs for debug.
         * @type {number}
         */
        get: function () { return Logs.DEBUG; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(LogLevel, Logs.DEBUG, {
        /**
         * Equals to `'DEBUG'`.
         * @type {string}
         */
        get: function () { return Logs[Logs.DEBUG]; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(LogLevel, "WEB_GROUP", {
        /**
         * Equals to `2`, logs for WebGroup module.
         * @type {number}
         */
        get: function () { return Logs.WEB_GROUP; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(LogLevel, Logs.WEB_GROUP, {
        /**
         * Equals to `'WEB_GROUP'`.
         * @type {string}
         */
        get: function () { return Logs[Logs.WEB_GROUP]; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(LogLevel, "WEBRTC", {
        /**
         * Equals to `3`, logs for WebRTCBuilder module.
         * @type {number}
         */
        get: function () { return Logs.WEBRTC; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(LogLevel, Logs.WEBRTC, {
        /**
         * Equals to `'WEBRTC'`.
         * @type {string}
         */
        get: function () { return Logs[Logs.WEBRTC]; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(LogLevel, "CHANNEL", {
        /**
         * Equals to `4`, logs for Channel module.
         * @type {number}
         */
        get: function () { return Logs.CHANNEL; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(LogLevel, Logs.CHANNEL, {
        /**
         * Equals to `'CHANNEL'`.
         * @type {string}
         */
        get: function () { return Logs[Logs.CHANNEL]; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(LogLevel, "TOPOLOGY", {
        /**
         * Equals to `5`, logs for Topology module.
         * @type {number}
         */
        get: function () { return Logs.TOPOLOGY; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(LogLevel, Logs.TOPOLOGY, {
        /**
         * Equals to `'TOPOLOGY'`.
         * @type {string}
         */
        get: function () { return Logs[Logs.TOPOLOGY]; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(LogLevel, "SIGNALING", {
        /**
         * Equals to `6`, logs for Signaling module.
         * @type {number}
         */
        get: function () { return Logs.SIGNALING; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(LogLevel, Logs.SIGNALING, {
        /**
         * Equals to `'SIGNALING'`.
         * @type {string}
         */
        get: function () { return Logs[Logs.SIGNALING]; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(LogLevel, "CHANNEL_BUILDER", {
        /**
         * Equals to `7`, logs for ChannelBuilder module.
         * @type {number}
         */
        get: function () { return Logs.CHANNEL_BUILDER; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(LogLevel, Logs.CHANNEL_BUILDER, {
        /**
         * Equals to `'CHANNEL_BUILDER'`.
         * @type {string}
         */
        get: function () { return Logs[Logs.CHANNEL_BUILDER]; },
        enumerable: true,
        configurable: true
    });
    return LogLevel;
}());
export { LogLevel };
/**
 * The options to be passed into {@link WebGroup} constructor.
 * @typedef {Object} WebGroupOptions
 * @property {Topology} [topology] Topology identifier
 * (Full mesh is the only one supported by Netflux for now).
 * @property {string} [signalingServer] Signaling URL for WebRTC.
 * @property {RTCConfiguration} [rtcConfiguration] Configuration for WebRTC.
 * @property {boolean} [autoRejoin] Whether to automatically rejoin the web group
 * on disconnect or not. Its value may be modified after {@link WebGroup}
 * instantiation at any time.
 */
/**
 * The options to be passed into {@link WebGroupBotServer} constructor.
 * @typedef {Object} WebGroupBotServerOptions
 * @property {Topology} [topology] See WebGroupOptions.topology
 * @property {string} [signalingServer] See WebGroupOptions.signalingServer
 * @property {RTCConfiguration} [rtcConfiguration] See WebGroupOptions.rtcConfiguration
 * @property {boolean} [autoRejoin] See WebGroupOptions.autoRejoin
 * @property {Object} bot Server related options of the bot.
 * @property {NodeJSHttpServer|NodeJSHttpsServer} bot.server NodeJS http(s) server.
 * @property {string} [bot.url] Bot server URL.
 * @property {boolean} [bot.perMessageDeflate] Enable/disable permessage-deflate.
 */
/**
 * @external {RTCConfiguration} https://developer.mozilla.org/en/docs/Web/API/RTCConfiguration
 */
/**
 * @external {Uint8Array} https://developer.mozilla.org/en/docs/Web/JavaScript/Reference/Global_Objects/Uint8Array
 */
/**
 * @external {NodeJSHttpServer} https://nodejs.org/api/http.html#http_class_http_server
 */
/**
 * @external {NodeJSHttpsServer} https://nodejs.org/api/https.html#https_class_https_server
 */
