var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
/**
 * Equals to true in any browser.
 */
export var isBrowser = (typeof global.window === 'undefined') ? false : true;
export function isOnline() {
    return isBrowser ? global.window.navigator.onLine : true;
}
export function isVisible() {
    return isBrowser ? global.window.document.visibilityState === 'visible' : true;
}
/**
 * Check whether the string is a valid URL.
 */
export function isURL(str) {
    var regex = '^' +
        // protocol identifier
        '(?:wss|ws)://' +
        // Host name/IP
        '[^\\s]+' +
        // port number
        '(?::\\d{2,5})?' +
        '$';
    return (new RegExp(regex, 'i')).test(str);
}
/**
 * Generate random key which will be used to join the network.
 */
export function generateKey() {
    var mask = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
    var length = 42; // Should be less then MAX_KEY_LENGTH value
    var values = randNumbers(length);
    var result = '';
    for (var i = 0; i < length; i++) {
        result += mask[values[i] % mask.length];
    }
    return result;
}
export function randNumbers(length) {
    if (length === void 0) { length = 1; }
    var res;
    if (isBrowser) {
        res = new Uint32Array(length);
        global.crypto.getRandomValues(res);
    }
    else {
        res = [];
        var bytes = crypto.randomBytes(4 * length);
        for (var i = 0; i < bytes.length; i += 4) {
            res[res.length] = bytes.readUInt32BE(i, true);
        }
    }
    return res;
}
export function equal(array1, array2) {
    return array1 !== undefined && array2 !== undefined
        && array1.length === array2.length
        && array1.every(function (v) { return array2.includes(v); });
}
export var MAX_KEY_LENGTH = 512;
var netfluxCSS = 'background-color: #FFCA28; padding: 0 3px';
var debugCSS = 'background-color: #b3ba2e; padding: 0 3px';
var webrtcCSS = 'background-color: #CE93D8; padding: 0 3px';
var channelCSS = 'background-color: #90CAF9; padding: 0 3px';
var topologyCSS = 'background-color: #26A69A; padding: 0 3px';
var signalingCSS = 'background-color: #66BB6A; padding: 0 3px';
var channelBuilderCSS = 'background-color: #F57C00; padding: 0 3px';
var signalingStateCSS = 'background-color: #9FA8DA; padding: 0 2px';
var webGroupStateCSS = 'background-color: #EF9A9A; padding: 0 2px';
var log = {
    webgroup: function () { },
    signalingState: function () { },
    webGroupState: function () { },
    webrtc: function () { },
    channel: function () { },
    topology: function () { },
    signaling: function () { },
    channelBuilder: function () { },
    debug: function () { },
};
export var LogLevel;
(function (LogLevel) {
    LogLevel[LogLevel["DEBUG"] = 0] = "DEBUG";
    LogLevel[LogLevel["WEB_GROUP"] = 1] = "WEB_GROUP";
    LogLevel[LogLevel["WEBRTC"] = 2] = "WEBRTC";
    LogLevel[LogLevel["CHANNEL"] = 3] = "CHANNEL";
    LogLevel[LogLevel["TOPOLOGY"] = 4] = "TOPOLOGY";
    LogLevel[LogLevel["SIGNALING"] = 5] = "SIGNALING";
    LogLevel[LogLevel["CHANNEL_BUILDER"] = 6] = "CHANNEL_BUILDER";
})(LogLevel || (LogLevel = {}));
export var logLevels = [];
export function setLogLevel(levels) {
    logLevels = levels;
    if (logLevels.includes(LogLevel.WEB_GROUP)) {
        log.webgroup = function (msg) {
            var rest = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                rest[_i - 1] = arguments[_i];
            }
            if (rest.length === 0) {
                console.info("%cNETFLUX WebGroup%c: " + msg, netfluxCSS, '');
            }
            else {
                console.info.apply(console, __spread(["%cNETFLUX WebGroup%c: " + msg, netfluxCSS, ''], rest));
            }
        };
        log.signalingState = function (msg, id) {
            console.info("%cNETFLUX " + id + " WebGroup%c: Signaling: %c" + msg + "%c", netfluxCSS, '', signalingStateCSS, '');
        };
        log.webGroupState = function (msg, id) {
            console.info("%cNETFLUX " + id + " WebGroup%c: WebGroup: %c" + msg + "%c", netfluxCSS, '', webGroupStateCSS, '');
        };
    }
    else {
        log.webgroup = function () { };
        log.signalingState = function () { };
        log.webGroupState = function () { };
    }
    // WebRTCBuilder logs
    if (logLevels.includes(LogLevel.WEBRTC)) {
        log.webrtc = function (msg) {
            var rest = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                rest[_i - 1] = arguments[_i];
            }
            if (rest.length === 0) {
                console.info("%cNETFLUX WebRTC%c: " + msg, webrtcCSS, '');
            }
            else {
                console.info.apply(console, __spread(["%cNETFLUX WebRTC%c: " + msg, webrtcCSS, ''], rest));
            }
        };
    }
    else {
        log.webrtc = function () { };
    }
    // Channel logs
    if (logLevels.includes(LogLevel.CHANNEL)) {
        log.channel = function (msg) {
            var rest = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                rest[_i - 1] = arguments[_i];
            }
            if (rest.length === 0) {
                console.info("%cNETFLUX Channel%c: " + msg, channelCSS, '');
            }
            else {
                console.info.apply(console, __spread(["%cNETFLUX Channel%c: " + msg, channelCSS, ''], rest));
            }
        };
    }
    else {
        log.channel = function () { };
    }
    // Topology logs
    if (logLevels.includes(LogLevel.TOPOLOGY)) {
        log.topology = function (msg) {
            var rest = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                rest[_i - 1] = arguments[_i];
            }
            if (rest.length === 0) {
                console.info("%cNETFLUX Topology%c: " + msg, topologyCSS, '');
            }
            else {
                console.info.apply(console, __spread(["%cNETFLUX Topology%c: " + msg, topologyCSS, ''], rest));
            }
        };
    }
    else {
        log.topology = function () { };
    }
    // Signaling logs
    if (logLevels.includes(LogLevel.SIGNALING)) {
        log.signaling = function (msg) {
            var rest = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                rest[_i - 1] = arguments[_i];
            }
            if (rest.length === 0) {
                console.info("%cNETFLUX Signaling%c: " + msg, signalingCSS, '');
            }
            else {
                console.info.apply(console, __spread(["%cNETFLUX Signaling%c: " + msg, signalingCSS, ''], rest));
            }
        };
    }
    else {
        log.signaling = function () { };
    }
    // Signaling logs
    if (logLevels.includes(LogLevel.CHANNEL_BUILDER)) {
        log.channelBuilder = function (msg) {
            var rest = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                rest[_i - 1] = arguments[_i];
            }
            if (rest.length === 0) {
                console.info("%cNETFLUX ChannelBuilder%c: " + msg, channelBuilderCSS, '');
            }
            else {
                console.info.apply(console, __spread(["%cNETFLUX ChannelBuilder%c: " + msg, channelBuilderCSS, ''], rest));
            }
        };
    }
    else {
        log.channelBuilder = function () { };
    }
    // Logs for current DEBUG
    if (logLevels.includes(LogLevel.DEBUG)) {
        log.debug = function (msg) {
            var rest = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                rest[_i - 1] = arguments[_i];
            }
            if (rest.length === 0) {
                console.info("%cNETFLUX Debug%c: " + msg, debugCSS, '');
            }
            else {
                console.info.apply(console, __spread(["%cNETFLUX Debug%c: " + msg, debugCSS, ''], rest));
            }
        };
    }
    else {
        log.debug = function () { };
    }
}
export { log };
