import { filter, map } from 'rxjs/operators';
import { service } from '../proto';
/**
 * Services are specific classes. Instance of such class communicates via
 * network with another instance of the same class. Indeed each peer in the
 * network instantiates its own service.
 * Each service has `.proto` file containing the desciption of its
 * communication protocol.
 */
var Service = /** @class */ (function () {
    function Service(id, protoMessage, serviceMessageSubject) {
        this.serviceId = id;
        this.protoMessage = protoMessage;
        if (serviceMessageSubject !== undefined) {
            this.setupServiceMessage(serviceMessageSubject);
        }
    }
    Service.encodeServiceMessage = function (serviceId, content) {
        return service.Message.encode(service.Message.create({
            id: serviceId,
            content: content,
        })).finish();
    };
    /**
     * Encode service message for sending over the network.
     *
     * @param msg Service specific message object
     */
    Service.prototype.encode = function (msg) {
        return service.Message.encode(service.Message.create({
            id: this.serviceId,
            content: this.protoMessage.encode(this.protoMessage.create(msg)).finish(),
        })).finish();
    };
    /**
     * Decode service message received from the network.
     *
     * @return  Service specific message object
     */
    Service.prototype.decode = function (bytes) {
        return this.protoMessage.decode(bytes);
    };
    Service.prototype.setupServiceMessage = function (serviceMessageSubject) {
        var _this = this;
        this.onServiceMessage = serviceMessageSubject.pipe(filter(function (_a) {
            var id = _a.id;
            return id === _this.serviceId;
        }), map(function (_a) {
            var channel = _a.channel, senderId = _a.senderId, recipientId = _a.recipientId, content = _a.content;
            return ({
                channel: channel,
                senderId: senderId,
                recipientId: recipientId,
                msg: _this.protoMessage.decode(content),
            });
        }));
    };
    return Service;
}());
export { Service };
