var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
import { Subject } from 'rxjs/Subject';
import { Channel } from '../Channel';
import { generateKey, isBrowser, isOnline, isURL, isVisible, log, MAX_KEY_LENGTH, randNumbers } from '../misc/Util';
import { Message, service, webChannel } from '../proto';
import { Signaling, SignalingState } from '../Signaling';
import { UserMessage } from '../UserMessage';
import { WebSocketBuilder } from '../WebSocketBuilder';
import { ChannelBuilder } from './ChannelBuilder';
import { Service } from './Service';
import { FullMesh } from './topology/FullMesh';
import { TopologyEnum, TopologyStateEnum } from './topology/Topology';
import { WebRTCBuilder } from './WebRTCBuilder';
/**
 * Service id.
 */
var ID = 200;
export var defaultOptions = {
    topology: TopologyEnum.FULL_MESH,
    signalingServer: 'wss://signaling.netflux.coedit.re',
    rtcConfiguration: {
        iceServers: [
            { urls: 'stun:stun3.l.google.com:19302' },
        ],
    },
    autoRejoin: true,
};
export var WebChannelState;
(function (WebChannelState) {
    WebChannelState[WebChannelState["JOINING"] = 0] = "JOINING";
    WebChannelState[WebChannelState["JOINED"] = 1] = "JOINED";
    WebChannelState[WebChannelState["LEAVING"] = 2] = "LEAVING";
    WebChannelState[WebChannelState["LEFT"] = 3] = "LEFT";
})(WebChannelState || (WebChannelState = {}));
/**
 * This class is an API starting point. It represents a group of collaborators
 * also called peers. Each peer can send/receive broadcast as well as personal
 * messages. Every peer in the `WebChannel` can invite another person to join
 * the `WebChannel` and he also possess enough information to be able to add it
 * preserving the current `WebChannel` structure (network topology).
 * [[include:installation.md]]
 */
var WebChannel = /** @class */ (function (_super) {
    __extends(WebChannel, _super);
    function WebChannel(_a) {
        var _b = _a === void 0 ? {} : _a, _c = _b.topology, topology = _c === void 0 ? defaultOptions.topology : _c, _d = _b.signalingServer, signalingServer = _d === void 0 ? defaultOptions.signalingServer : _d, _e = _b.rtcConfiguration, rtcConfiguration = _e === void 0 ? defaultOptions.rtcConfiguration : _e, _f = _b.autoRejoin, autoRejoin = _f === void 0 ? defaultOptions.autoRejoin : _f;
        var _this = _super.call(this, ID, webChannel.Message) || this;
        // PUBLIC MEMBERS
        _this.topology = topology;
        _this.id = _this.generateId();
        _this.myId = _this.generateId();
        _this.members = [_this.myId];
        _this.key = '';
        _this.autoRejoin = autoRejoin;
        // PUBLIC EVENT HANDLERS
        _this.onMemberJoin = function none() { };
        _this.onMemberLeave = function none() { };
        _this.onMessage = function none() { };
        _this.onStateChange = function none() { };
        _this.onSignalingStateChange = function none() { };
        // PRIVATE
        _this.state = WebChannelState.LEFT;
        _this.userMsg = new UserMessage();
        // Signaling init
        _this.signaling = new Signaling(_this, signalingServer);
        _this.signaling.onChannel.subscribe(function (ch) { return _this.initChannel(ch); });
        _this.signaling.onState.subscribe(function (state) {
            log.signalingState(SignalingState[state], _this.myId);
            _this.onSignalingStateChange(state);
            switch (state) {
                case SignalingState.CONNECTING:
                    _this.setState(WebChannelState.JOINING);
                    break;
                case SignalingState.CLOSED:
                    if (_this.topologyService.state === TopologyStateEnum.DISCONNECTED) {
                        _this.setState(WebChannelState.LEFT);
                    }
                    _this.rejoin(3000);
                    break;
                case SignalingState.STABLE:
                    _this.topologyService.setStable();
                    break;
            }
        });
        // Services init
        _this.serviceMessageSubject = new Subject();
        _super.prototype.setupServiceMessage.call(_this, _this.serviceMessageSubject);
        _this.webRTCBuilder = new WebRTCBuilder(_this, rtcConfiguration);
        _this.webSocketBuilder = new WebSocketBuilder(_this);
        _this.channelBuilder = new ChannelBuilder(_this);
        _this.onServiceMessage.subscribe(function (msg) { return _this.treatServiceMessage(msg); }, function (err) { return console.error('service/WebChannel inner message error', err); });
        // Topology init
        _this.setTopology(topology);
        // FIXME: manage topologyService onState subscription
        // Listen to browser events only
        if (isBrowser) {
            // global.window.addEventListener('offline', () => {
            //   setTimeout(() => {
            //     if (!isOnline()) {
            //       this.internalLeave()
            //     }
            //   }, 2000)
            // })
            global.window.addEventListener('online', function () {
                if (isVisible() && _this.state === WebChannelState.LEFT) {
                    _this.rejoin();
                }
            });
            global.window.addEventListener('visibilitychange', function () {
                if (isVisible() && _this.state === WebChannelState.LEFT) {
                    _this.rejoin();
                }
            });
            global.window.addEventListener('beforeunload', function () { return _this.internalLeave(); });
        }
        return _this;
    }
    /**
     * Join the network via a key provided by one of the network member or a `Channel`.
     */
    WebChannel.prototype.join = function (key) {
        if (key === void 0) { key = generateKey(); }
        if (typeof key !== 'string') {
            throw new Error("Failed to join: the key type \"" + typeof key + "\" is not a \"string\"");
        }
        else if (key === '') {
            throw new Error('Failed to join: the key is an empty string');
        }
        else if (key.length > MAX_KEY_LENGTH) {
            throw new Error("Failed to join : the key length of " + key.length + " exceeds the maximum of " + MAX_KEY_LENGTH + " characters");
        }
        this.key = key;
        if (isOnline() && this.state === WebChannelState.LEFT && this.signaling.state === SignalingState.CLOSED) {
            this.setState(WebChannelState.JOINING);
            this.isRejoinDisabled = !this.autoRejoin;
            this.signaling.join(this.key);
        }
    };
    /**
     * Invite a server peer to join the network.
     */
    WebChannel.prototype.invite = function (url) {
        var _this = this;
        if (isURL(url)) {
            this.webSocketBuilder.connect(url + "/invite?wcId=" + this.id + "&senderId=" + this.myId)
                .then(function (connection) { return _this.initChannel(new Channel(_this, connection)); })
                .catch(function (err) { return console.error("Failed to invite the bot " + url + ": " + err.message); });
        }
        else {
            throw new Error("Failed to invite a bot: " + url + " is not a valid URL");
        }
    };
    /**
     * Leave the network which means close channels with all peers and connection
     * with Signaling server.
     */
    WebChannel.prototype.leave = function () {
        if (this.state !== WebChannelState.LEAVING && this.state !== WebChannelState.LEFT) {
            this.setState(WebChannelState.LEAVING);
            this.key = '';
            this.isRejoinDisabled = true;
            this.signaling.close();
            this.topologyService.leave();
        }
    };
    /**
     * Broadcast a message to the network.
     */
    WebChannel.prototype.send = function (data) {
        if (this.members.length !== 1) {
            var msg = {
                senderId: this.myId,
                recipientId: 0,
                isService: false,
            };
            var chunkedData = this.userMsg.encode(data);
            try {
                for (var chunkedData_1 = __values(chunkedData), chunkedData_1_1 = chunkedData_1.next(); !chunkedData_1_1.done; chunkedData_1_1 = chunkedData_1.next()) {
                    var chunk = chunkedData_1_1.value;
                    msg.content = chunk;
                    this.topologyService.send(msg);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (chunkedData_1_1 && !chunkedData_1_1.done && (_a = chunkedData_1.return)) _a.call(chunkedData_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
        }
        var e_1, _a;
    };
    /**
     * Send a message to a particular peer in the network.
     */
    WebChannel.prototype.sendTo = function (id, data) {
        if (this.members.length !== 1) {
            var msg = {
                senderId: this.myId,
                recipientId: id,
                isService: false,
            };
            var chunkedData = this.userMsg.encode(data);
            try {
                for (var chunkedData_2 = __values(chunkedData), chunkedData_2_1 = chunkedData_2.next(); !chunkedData_2_1.done; chunkedData_2_1 = chunkedData_2.next()) {
                    var chunk = chunkedData_2_1.value;
                    msg.content = chunk;
                    this.topologyService.sendTo(msg);
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (chunkedData_2_1 && !chunkedData_2_1.done && (_a = chunkedData_2.return)) _a.call(chunkedData_2);
                }
                finally { if (e_2) throw e_2.error; }
            }
        }
        var e_2, _a;
    };
    WebChannel.prototype.onMemberJoinProxy = function (id) {
        if (!this.members.includes(id)) {
            this.members[this.members.length] = id;
            this.onMemberJoin(id);
        }
    };
    WebChannel.prototype.onMemberLeaveProxy = function (id) {
        if (this.members.includes(id)) {
            this.members.splice(this.members.indexOf(id), 1);
            this.onMemberLeave(id);
        }
    };
    /**
     * Send service message to a particular peer in the network.
     */
    WebChannel.prototype.sendToProxy = function (_a) {
        var _b = _a === void 0 ? {} : _a, _c = _b.senderId, senderId = _c === void 0 ? this.myId : _c, _d = _b.recipientId, recipientId = _d === void 0 ? this.myId : _d, _e = _b.isService, isService = _e === void 0 ? true : _e, content = _b.content;
        this.topologyService.sendTo({ senderId: senderId, recipientId: recipientId, isService: isService, content: content });
    };
    /**
     * Broadcast service message to the network.
     */
    WebChannel.prototype.sendProxy = function (_a) {
        var _b = _a === void 0 ? {} : _a, _c = _b.senderId, senderId = _c === void 0 ? this.myId : _c, _d = _b.recipientId, recipientId = _d === void 0 ? 0 : _d, _e = _b.isService, isService = _e === void 0 ? true : _e, content = _b.content;
        this.topologyService.send({ senderId: senderId, recipientId: recipientId, isService: isService, content: content });
    };
    WebChannel.prototype.encode = function (_a) {
        var _b = _a === void 0 ? {} : _a, _c = _b.senderId, senderId = _c === void 0 ? this.myId : _c, _d = _b.recipientId, recipientId = _d === void 0 ? 0 : _d, _e = _b.isService, isService = _e === void 0 ? true : _e, content = _b.content;
        var msg = { senderId: senderId, recipientId: recipientId, isService: isService, content: content };
        return Message.encode(Message.create(msg)).finish();
    };
    /**
     * Message handler. All messages arrive here first.
     */
    WebChannel.prototype.onMessageProxy = function (channel, bytes) {
        var msg = Message.decode(new Uint8Array(bytes));
        // recipientId values:
        // 0: broadcast message
        // 1: is a message to me from a peer who does not know yet my ID
        if (msg.recipientId === 0 || msg.recipientId === this.myId || msg.recipientId === 1) {
            this.treatMessage(channel, msg);
        }
        if (msg.recipientId !== this.myId) {
            this.topologyService.forward(msg);
        }
    };
    WebChannel.prototype.treatMessage = function (channel, msg) {
        // User Message
        if (!msg.isService) {
            var data = this.userMsg.decode(msg.content, msg.senderId);
            if (data !== undefined) {
                this.onMessage(msg.senderId, data);
            }
            // Service Message
        }
        else {
            var fullMsg = service.Message.decode(msg.content);
            fullMsg.channel = channel;
            fullMsg.senderId = msg.senderId;
            fullMsg.recipientId = msg.recipientId;
            this.serviceMessageSubject.next(fullMsg);
        }
    };
    WebChannel.prototype.treatServiceMessage = function (_a) {
        var channel = _a.channel, senderId = _a.senderId, msg = _a.msg;
        switch (msg.type) {
            case 'init': {
                var _b = msg.init, topology = _b.topology, wcId = _b.wcId, generatedIds = _b.generatedIds, members = _b.members;
                // Check whether the intermidiary peer is already a member of your
                // network (possible when merging two networks (works with FullMesh)).
                // If it is a case then you are already a member of the network.
                if (this.members.includes(senderId)) {
                    if (!generatedIds.includes(this.myId)) {
                        this.setState(WebChannelState.LEAVING);
                        console.warn("Failed merge networks: my members contain intermediary peer id,\n            but my id is not included into the intermediary peer members");
                        channel.closeQuietly();
                        this.topologyService.leave();
                        return;
                    }
                    if (this.topology !== topology) {
                        this.setState(WebChannelState.LEAVING);
                        console.warn('Failed merge networks: different topologies');
                        channel.closeQuietly();
                        this.topologyService.leave();
                        return;
                    }
                    log.webgroup("I:" + this.myId + " close connection with intermediary member " + senderId + ",\n          because already connected with him");
                    this.setState(WebChannelState.JOINED);
                    channel.closeQuietly();
                }
                else {
                    this.setTopology(topology);
                    this.id = wcId;
                    channel.id = senderId;
                    channel.send(this.encode({ recipientId: channel.id, content: _super.prototype.encode.call(this, { initOk: true }) }));
                    this.topologyService.initJoining(channel, members);
                }
                break;
            }
            case 'initOk': {
                channel.id = senderId;
                this.topologyService.addJoining(channel);
                break;
            }
            default:
                throw new Error("Unknown message type: \"" + msg.type + "\"");
        }
    };
    WebChannel.prototype.setState = function (state) {
        if (this.state !== state) {
            log.webGroupState(WebChannelState[state], this.myId);
            this.state = state;
            this.onStateChange(state);
        }
    };
    /**
     * Delegate adding a new peer in the network to topology.
     */
    WebChannel.prototype.initChannel = function (ch) {
        var msg = this.encode({
            recipientId: 1,
            content: _super.prototype.encode.call(this, { init: {
                    topology: this.topology,
                    wcId: this.id,
                    generatedIds: this.members,
                    members: this.members,
                } }),
        });
        ch.send(msg);
    };
    WebChannel.prototype.setTopology = function (topology) {
        if (this.topologyService !== undefined) {
            if (this.topology !== topology) {
                this.topology = topology;
                this.topologyService.leave();
                this.topologyService = new FullMesh(this);
                this.subscribeToTopology();
            }
        }
        else {
            this.topology = topology;
            this.topologyService = new FullMesh(this);
            this.subscribeToTopology();
        }
    };
    WebChannel.prototype.subscribeToTopology = function () {
        var _this = this;
        if (this.topologySub) {
            this.topologySub.unsubscribe();
        }
        this.topologySub = this.topologyService.onState.subscribe(function (state) {
            switch (state) {
                case TopologyStateEnum.JOINING:
                    _this.setState(WebChannelState.JOINING);
                    break;
                case TopologyStateEnum.JOINED:
                    _this.setState(WebChannelState.JOINED);
                    break;
                case TopologyStateEnum.STABLE:
                    _this.setState(WebChannelState.JOINED);
                    _this.signaling.open();
                    break;
                case TopologyStateEnum.DISCONNECTED:
                    if (_this.signaling.state === SignalingState.CLOSED) {
                        _this.setState(WebChannelState.LEFT);
                    }
                    _this.rejoin(2000);
                    break;
            }
        });
    };
    WebChannel.prototype.rejoin = function (timeout) {
        var _this = this;
        if (timeout === void 0) { timeout = 0; }
        if (!this.isRejoinDisabled) {
            this.isRejoinDisabled = !this.autoRejoin;
            global.clearTimeout(this.rejoinTimer);
            this.rejoinTimer = setTimeout(function () {
                if (isOnline() && isVisible()) {
                    log.webgroup('REJOIN...');
                    _this.signaling.join(_this.key);
                }
            }, timeout);
        }
    };
    /**
     * Generate random id for a `WebChannel` or a new peer.
     */
    WebChannel.prototype.generateId = function (excludeIds) {
        if (excludeIds === void 0) { excludeIds = []; }
        var id = randNumbers()[0];
        if (excludeIds.includes(id)) {
            return this.generateId();
        }
        return id;
    };
    WebChannel.prototype.internalLeave = function () {
        if (this.state !== WebChannelState.LEAVING && this.state !== WebChannelState.LEFT) {
            this.signaling.close();
            this.topologyService.leave();
            this.setState(WebChannelState.LEFT);
        }
    };
    return WebChannel;
}(Service));
export { WebChannel };
