import { Observable } from 'rxjs/Observable';
import { Channel } from '../Channel';
import { signaling as sigProto } from '../proto';
import { IWebRTCStream } from '../Signaling';
import { Service } from './Service';
import { WebChannel } from './WebChannel';
export declare const CONNECT_TIMEOUT = 20000;
export interface ISignalingConnection {
    onMessage: Observable<any>;
    send: (msg: sigProto.IContent) => void;
}
/**
 * Service class responsible to establish `RTCDataChannel` between two remotes via
 * signaling server or `WebChannel`.
 *
 */
export declare class WebRTCBuilder extends Service {
    /**
     * Indicates whether WebRTC is supported by the environment.
     */
    static readonly isSupported: boolean;
    private wc;
    private rtcConfiguration;
    constructor(wc: WebChannel, rtcConfiguration: RTCConfiguration);
    /**
     * Listen on `RTCDataChannel` from WebChannel (another peer is playing a signaling role).
     * Starts to listen on **SDP answer**.
     */
    onChannelFromWebChannel(): Observable<Channel>;
    /**
     * Establish an `RTCDataChannel` with a peer identified by `id` trough `WebChannel`.
     * Starts by sending an **SDP offer**.
     *
     * @param id  Peer id
     */
    connectOverWebChannel(id: number): Promise<Channel>;
    /**
     * Listen on `RTCDataChannel` from Signaling server.
     * Starts to listen on **SDP answer**.
     */
    onChannelFromSignaling(wrtcStream: IWebRTCStream): Observable<Channel>;
    /**
     * Establish an `RTCDataChannel` with a peer identified by `id` trough Signaling server.
     * Starts by sending an **SDP offer**.
     */
    connectOverSignaling(wrtcStream: IWebRTCStream): Promise<Channel>;
    private establishChannel(onMessage, send, id?);
    private onChannel(onMessage, send);
    private setupLocalCandidates(pc, cb);
    private openChannel(pc, id?);
}
