var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
import { Observable } from 'rxjs/Observable';
import { filter, map, pluck } from 'rxjs/operators';
import { ReplaySubject } from 'rxjs/ReplaySubject';
import { Channel } from '../Channel';
import { log } from '../misc/Util';
import { webRTCBuilder } from '../proto';
import { Service } from './Service';
var counter = 0;
/**
 * Service id.
 */
var ID = 300;
export var CONNECT_TIMEOUT = 20000;
/**
 * Service class responsible to establish `RTCDataChannel` between two remotes via
 * signaling server or `WebChannel`.
 *
 */
var WebRTCBuilder = /** @class */ (function (_super) {
    __extends(WebRTCBuilder, _super);
    function WebRTCBuilder(wc, rtcConfiguration) {
        var _this = _super.call(this, ID, webRTCBuilder.Message, wc.serviceMessageSubject) || this;
        _this.wc = wc;
        _this.rtcConfiguration = rtcConfiguration;
        return _this;
    }
    Object.defineProperty(WebRTCBuilder, "isSupported", {
        /**
         * Indicates whether WebRTC is supported by the environment.
         */
        get: function () {
            return global.RTCPeerConnection !== undefined;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Listen on `RTCDataChannel` from WebChannel (another peer is playing a signaling role).
     * Starts to listen on **SDP answer**.
     */
    WebRTCBuilder.prototype.onChannelFromWebChannel = function () {
        var _this = this;
        if (WebRTCBuilder.isSupported) {
            return this.onChannel(this.onServiceMessage.pipe(filter(function (_a) {
                var msg = _a.msg;
                return msg.isInitiator;
            }), map(function (_a) {
                var msg = _a.msg, senderId = _a.senderId;
                msg.id = senderId;
                return msg;
            })), function (msg, id) { return _this.wc.sendToProxy({ recipientId: id, content: _super.prototype.encode.call(_this, msg) }); });
        }
        log.webrtc('WebRTC is not supported');
        throw new Error('WebRTC is not supported');
    };
    /**
     * Establish an `RTCDataChannel` with a peer identified by `id` trough `WebChannel`.
     * Starts by sending an **SDP offer**.
     *
     * @param id  Peer id
     */
    WebRTCBuilder.prototype.connectOverWebChannel = function (id) {
        var _this = this;
        if (WebRTCBuilder.isSupported) {
            return this.establishChannel(this.onServiceMessage.pipe(filter(function (_a) {
                var msg = _a.msg, senderId = _a.senderId;
                return senderId === id && !msg.isInitiator;
            }), pluck('msg')), function (msg) {
                msg.isInitiator = true;
                _this.wc.sendToProxy({ recipientId: id, content: _super.prototype.encode.call(_this, msg) });
            }, id);
        }
        throw new Error('WebRTC is not supported');
    };
    /**
     * Listen on `RTCDataChannel` from Signaling server.
     * Starts to listen on **SDP answer**.
     */
    WebRTCBuilder.prototype.onChannelFromSignaling = function (wrtcStream) {
        var _this = this;
        if (WebRTCBuilder.isSupported) {
            return this.onChannel(wrtcStream.message.pipe(filter(function (msg) { return msg.id !== 0; }), map(function (msg) {
                if (msg.type === 'data') {
                    var data = _super.prototype.decode.call(_this, msg.data);
                    data.id = msg.id;
                    return data;
                }
                return msg;
            })), function (msg, id) {
                if (!('isError' in msg || 'isEnd' in msg)) {
                    msg.data = webRTCBuilder.Message.encode(webRTCBuilder.Message.create(msg)).finish();
                }
                msg.id = id;
                wrtcStream.send(msg);
            });
        }
        throw new Error('WebRTC is not supported');
    };
    /**
     * Establish an `RTCDataChannel` with a peer identified by `id` trough Signaling server.
     * Starts by sending an **SDP offer**.
     */
    WebRTCBuilder.prototype.connectOverSignaling = function (wrtcStream) {
        var _this = this;
        if (WebRTCBuilder.isSupported) {
            return this.establishChannel(wrtcStream.message.pipe(filter(function (msg) { return msg.id === 0; }), map(function (msg) { return msg.type === 'data' ? _super.prototype.decode.call(_this, msg.data) : msg; })), function (msg) {
                if (!('isError' in msg || 'isEnd' in msg)) {
                    wrtcStream.send({ data: webRTCBuilder.Message.encode(webRTCBuilder.Message.create(msg)).finish() });
                }
                else {
                    wrtcStream.send(msg);
                }
            });
        }
        throw new Error('WebRTC is not supported');
    };
    WebRTCBuilder.prototype.establishChannel = function (onMessage, send, id) {
        var _this = this;
        if (id === void 0) { id = 1; }
        var isOfferSent = false;
        var pc = new global.RTCPeerConnection(this.rtcConfiguration);
        pc.onicegatheringstatechange = function () {
            log.webrtc('ICE GATHERING STATE changed to', pc.iceGatheringState);
            if (pc.iceGatheringState === 'complete' && isOfferSent) {
                send({ isEnd: true });
                isOfferSent = false;
            }
        };
        counter++;
        pc.onsignalingstatechange = function () { log.webrtc('SIGNALING STATE changed to', pc.signalingState); };
        var rcs = new ReplaySubject();
        this.setupLocalCandidates(pc, function (iceCandidate) { return send({ iceCandidate: iceCandidate }); });
        return new Promise(function (resolve, reject) {
            var onMessageSub = onMessage.subscribe(function (_a) {
                var answer = _a.answer, iceCandidate = _a.iceCandidate, isError = _a.isError, isEnd = _a.isEnd;
                pc.oniceconnectionstatechange = function () {
                    log.webrtc(counter + ': ICE CONNECTION STATE changed to', pc.iceConnectionState);
                    if (pc.iceConnectionState === 'failed') {
                        rcs.complete();
                        pc.close();
                        send({ isError: true });
                        onMessageSub.unsubscribe();
                        reject(new Error(counter + ': Failed to establish RTCDataChannel: Ice Connection failed'));
                    }
                };
                if (isError) {
                    rcs.complete();
                    onMessageSub.unsubscribe();
                    if (pc.iceConnectionState !== 'connected' && pc.iceConnectionState !== 'completed') {
                        log.webrtc(counter + ': Failed to establish RTCDataChannel: remote peer error');
                        pc.close();
                        reject(new Error(counter + ': Remote peer error'));
                    }
                    else {
                        log.webrtc(counter + ': Remote peer error, but RTCDataChannel has still been established');
                    }
                }
                else if (answer) {
                    log.webrtc(counter + ': REMOTE Answer is received', { answer: answer });
                    pc.setRemoteDescription({ type: 'answer', sdp: answer })
                        .then(function () {
                        rcs.subscribe(function (ic) { return pc.addIceCandidate(ic)
                            .catch(function (err) { return log.webrtc('${id}: Failed to add REMOTE Ice Candidate', err); }); });
                    })
                        .catch(function (err) {
                        log.webrtc(counter + ': Failed to establish RTCDataChannel: Set REMOTE Answer ERROR', err);
                        rcs.complete();
                        pc.close();
                        send({ isError: true });
                        onMessageSub.unsubscribe();
                        reject(new Error(counter + ': Error during setting a remote answer'));
                    });
                }
                else if (iceCandidate) {
                    if (iceCandidate.candidate !== '') {
                        log.webrtc(counter + ': REMOTE Ice Candidate is received', iceCandidate);
                        rcs.next(new global.RTCIceCandidate(iceCandidate));
                    }
                    else {
                        log.webrtc(counter + ': REMOTE Ice Candidate gathering COMPLETED', iceCandidate.candidate);
                        rcs.complete();
                    }
                }
                else if (isEnd) {
                    log.webrtc(counter + ': REMOTE Peer FINISHED send all data');
                    onMessageSub.unsubscribe();
                }
                else {
                    log.webrtc(counter + ': Unknown message from a remote peer: stopping connection establishment');
                    rcs.complete();
                    pc.close();
                    send({ isError: true });
                    onMessageSub.unsubscribe();
                    reject(new Error(counter + ': Unknown message from a remote peer'));
                }
            }, function (err) {
                log.webrtc(counter + ': Intermidiary steram was interrupted', err);
                rcs.complete();
                if (pc.iceConnectionState !== 'connected' && pc.iceConnectionState !== 'completed') {
                    log.webrtc(counter + ': Failed to establish RTCDataChannel: Intermidiary steram was interrupted');
                    pc.close();
                    reject(err);
                }
                else {
                    log.webrtc(counter + ': Intermidiary steram was interrupted, but RTCDataChannel has still been established');
                }
            }, function () {
                rcs.complete();
                if (pc.iceConnectionState !== 'connected' && pc.iceConnectionState !== 'completed') {
                    var err = new Error('Intermidiary steram was closed');
                    log.webrtc(counter + ': Failed to establish RTCDataChannel', err);
                    pc.close();
                    reject(err);
                }
                else {
                    log.webrtc(counter + ': Connection with Signaling closed, but RTCDataChannel has still been established');
                }
            });
            _this.openChannel(pc, id)
                .then(resolve)
                .catch(function (err) {
                rcs.complete();
                pc.close();
                send({ isError: true });
                onMessageSub.unsubscribe();
                reject(err);
            });
            pc.createOffer()
                .then(function (offer) { return pc.setLocalDescription(offer); })
                .then(function () {
                if (pc.localDescription && pc.localDescription.sdp) {
                    send({ offer: pc.localDescription.sdp });
                    isOfferSent = true;
                    if (pc.iceGatheringState === 'complete') {
                        send({ isEnd: true });
                    }
                }
            })
                .catch(function (err) {
                rcs.complete();
                pc.close();
                send({ isError: true });
                onMessageSub.unsubscribe();
                log.webrtc(counter + ': Failed to establish RTCDataChannel: Error while setting LOCAL Offer', err);
                reject(err);
            });
        });
    };
    WebRTCBuilder.prototype.onChannel = function (onMessage, send) {
        var _this = this;
        var remotes = new Map();
        var failedRemotes = [];
        return Observable.create(function (observer) {
            onMessage.pipe(filter(function (_a) {
                var id = _a.id;
                return !failedRemotes.includes(id);
            }))
                .subscribe(function (msg) {
                var offer = msg.offer, iceCandidate = msg.iceCandidate, id = msg.id, isError = msg.isError, isEnd = msg.isEnd;
                var clean = function (peerId, pc, rcs) {
                    pc.oniceconnectionstatechange = function () { };
                    rcs.complete();
                    send({ isError: true }, peerId);
                    remotes.delete(id);
                    failedRemotes[failedRemotes.length] = peerId;
                };
                if (isError) {
                    log.webrtc(id + ": Failed to establish RTCDataChannel: remote peer error");
                    var remote = remotes.get(id);
                    if (remote) {
                        remote[1].complete();
                        remotes.delete(id);
                    }
                    failedRemotes[failedRemotes.length] = id;
                }
                else {
                    var pc_1;
                    var rcs_1;
                    var isAnswerSent_1;
                    var remote = remotes.get(id);
                    if (remote) {
                        _a = __read(remote, 3), pc_1 = _a[0], rcs_1 = _a[1], isAnswerSent_1 = _a[2];
                    }
                    else {
                        isAnswerSent_1 = false;
                        pc_1 = new global.RTCPeerConnection(_this.rtcConfiguration);
                        pc_1.oniceconnectionstatechange = function () {
                            log.webrtc(id + ": ICE CONNECTION STATE changed to", pc_1.iceConnectionState);
                            if (pc_1.iceConnectionState === 'failed') {
                                clean(id, pc_1, rcs_1);
                            }
                        };
                        pc_1.onicegatheringstatechange = function () {
                            log.webrtc(id + ": ICE GATHERING STATE changed to", pc_1.iceGatheringState);
                            if (pc_1.iceGatheringState === 'complete' && isAnswerSent_1) {
                                send({ isEnd: true }, id);
                                pc_1.onicegatheringstatechange = function () { };
                            }
                        };
                        pc_1.onsignalingstatechange = function () { log.webrtc(id + ": SIGNALING STATE changed to", pc_1.signalingState); };
                        rcs_1 = new ReplaySubject();
                        _this.setupLocalCandidates(pc_1, function (ic) { return send({ iceCandidate: ic }, id); });
                        _this.openChannel(pc_1)
                            .then(function (ch) {
                            pc_1.oniceconnectionstatechange = function () { };
                            observer.next(ch);
                        })
                            .catch(function () { return clean(id, pc_1, rcs_1); });
                        remotes.set(id, [pc_1, rcs_1, isAnswerSent_1]);
                    }
                    if (offer) {
                        log.webrtc(id + ": REMOTE OFFER is received", { offer: offer });
                        pc_1.setRemoteDescription({ type: 'offer', sdp: offer })
                            .then(function () { return rcs_1.subscribe(function (ic) { return pc_1.addIceCandidate(ic).catch(function (err) { return log.webrtc(id + ": Failed to addIceCandidate", err); }); }); })
                            .then(function () { return pc_1.createAnswer(); })
                            .then(function (answer) { return pc_1.setLocalDescription(answer); })
                            .then(function () {
                            if (pc_1.localDescription && pc_1.localDescription.sdp) {
                                log.webrtc(id + ": Send LOCAL ANSWER", { answer: pc_1.localDescription.sdp });
                                send({ answer: pc_1.localDescription.sdp }, id);
                                isAnswerSent_1 = true;
                                if (pc_1.iceGatheringState === 'complete') {
                                    send({ isEnd: true }, id);
                                    pc_1.onicegatheringstatechange = function () { };
                                }
                            }
                        })
                            .catch(function (err) {
                            log.webrtc(id + ": Error during offer/answer setting", err);
                            clean(id, pc_1, rcs_1);
                        });
                    }
                    else if (iceCandidate) {
                        if (iceCandidate.candidate !== '') {
                            log.webrtc(id + ": REMOTE Ice Candidate is received", iceCandidate);
                            rcs_1.next(new global.RTCIceCandidate(iceCandidate));
                        }
                        else {
                            log.webrtc(id + ": REMOTE Ice Candidate gathering COMPLETED", iceCandidate.candidate);
                            rcs_1.complete();
                        }
                    }
                    else if (isEnd) {
                        log.webrtc(id + ": REMOTE Peer FINISHED send all data");
                        remotes.delete(id);
                    }
                    else {
                        log.webrtc(id + ": Unknown message from a remote peer: stopping connection establishment", msg);
                        clean(id, pc_1, rcs_1);
                    }
                }
                var _a;
            }, function (err) {
                log.webrtc('Intermidiary steram was interrupted', err);
                try {
                    for (var remotes_1 = __values(remotes), remotes_1_1 = remotes_1.next(); !remotes_1_1.done; remotes_1_1 = remotes_1.next()) {
                        var _a = __read(remotes_1_1.value, 2), id = _a[0], _b = __read(_a[1], 2), pc = _b[0], rcs = _b[1];
                        rcs.complete();
                        if (pc.iceConnectionState !== 'connected' && pc.iceConnectionState !== 'completed') {
                            log.webrtc(id + ": Failed to establish RTCDataChannel");
                            pc.close();
                        }
                        else {
                            log.webrtc(id + ": RTCDataChannel with " + id + " has still been established");
                        }
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (remotes_1_1 && !remotes_1_1.done && (_c = remotes_1.return)) _c.call(remotes_1);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
                observer.error(err);
                var e_1, _c;
            }, function () {
                log.webrtc('${id}: Intermidiary stream has closed');
                try {
                    for (var remotes_2 = __values(remotes), remotes_2_1 = remotes_2.next(); !remotes_2_1.done; remotes_2_1 = remotes_2.next()) {
                        var _a = __read(remotes_2_1.value, 2), id = _a[0], _b = __read(_a[1], 2), pc = _b[0], rcs = _b[1];
                        rcs.complete();
                        if (pc.iceConnectionState !== 'connected' && pc.iceConnectionState !== 'completed') {
                            log.webrtc(id + ": Failed to establish RTCDataChannel: Intermidiary stream has closed");
                            pc.close();
                        }
                        else {
                            log.webrtc(id + ": RTCDataChannel has still been established");
                        }
                    }
                }
                catch (e_2_1) { e_2 = { error: e_2_1 }; }
                finally {
                    try {
                        if (remotes_2_1 && !remotes_2_1.done && (_c = remotes_2.return)) _c.call(remotes_2);
                    }
                    finally { if (e_2) throw e_2.error; }
                }
                observer.complete();
                var e_2, _c;
            });
        });
    };
    WebRTCBuilder.prototype.setupLocalCandidates = function (pc, cb) {
        pc.onicecandidate = function (evt) {
            if (evt.candidate !== null) {
                log.webrtc('LOCAL Ice Candidate gathered', evt.candidate.candidate);
                cb({
                    candidate: evt.candidate.candidate,
                    sdpMid: evt.candidate.sdpMid,
                    sdpMLineIndex: evt.candidate.sdpMLineIndex,
                });
            }
        };
    };
    WebRTCBuilder.prototype.openChannel = function (pc, id) {
        var _this = this;
        var dc;
        if (id) {
            try {
                dc = pc.createDataChannel((this.wc.myId).toString());
            }
            catch (err) {
                log.webrtc('Failed to create RTCDataChannel', err);
                return Promise.reject(err);
            }
            return new Promise(function (resolve, reject) {
                var timeout = setTimeout(function () {
                    if (dc.readyState !== 'open') {
                        dc.close();
                        var errMsg = "RTCDataChannel " + CONNECT_TIMEOUT + "ms connection timeout with '" + id + "'";
                        log.webrtc(errMsg);
                        reject(new Error(errMsg));
                    }
                }, CONNECT_TIMEOUT);
                dc.onopen = function () {
                    clearTimeout(timeout);
                    log.webrtc("RTCDataChannel with " + id + " has opened", pc.sctp);
                    resolve(new Channel(_this.wc, dc, { rtcPeerConnection: pc, id: id }));
                };
            });
        }
        else {
            return new Promise(function (resolve, reject) {
                var timeout = setTimeout(function () {
                    if (dc === undefined || dc.readyState !== 'open') {
                        if (dc !== undefined) {
                            dc.close();
                        }
                        var errMsg = "RTCDataChannel " + CONNECT_TIMEOUT + "ms connection timeout with joining peer";
                        log.webrtc(errMsg);
                        reject(new Error(errMsg));
                    }
                }, CONNECT_TIMEOUT);
                pc.ondatachannel = function (dcEvt) {
                    dc = dcEvt.channel;
                    var peerId = Number.parseInt(dc.label, 10);
                    dc.onopen = function () {
                        clearTimeout(timeout);
                        log.webrtc("RTCDataChannel with " + peerId + " has opened", pc.sctp);
                        resolve(new Channel(_this.wc, dc, { rtcPeerConnection: pc, id: peerId }));
                    };
                };
            });
        }
    };
    return WebRTCBuilder;
}(Service));
export { WebRTCBuilder };
