import { Observable } from 'rxjs/Observable';
import { Channel } from '../../Channel';
import { IMessage } from '../../proto';
import { Service } from '../Service';
import { WebChannel } from '../WebChannel';
import { ITopology, TopologyStateEnum } from './Topology';
/**
 * {@link FullMesh} identifier.
 * @ignore
 * @type {number}
 */
export declare const FULL_MESH = 3;
/**
 * Fully connected web channel manager. Implements fully connected topology
 * network, when each peer is connected to each other.
 *
 */
export declare class FullMesh extends Service implements ITopology {
    readonly heartbeat: Uint8Array;
    private wc;
    /**
     * Directly connected peers.
     */
    private adjacentMembers;
    /**
     * Peers that are not adjacent. When the connection with a distant peer is established,
     * his id is removed from this map a new entry is added to the `adjacentMembers` property.
     */
    private distantMembers;
    /**
     * Set of peers among `distantMembers` to whom you are connecting right now.
     */
    private connectingMembers;
    private stateSubject;
    private _state;
    private heartbeatInterval;
    private membersRequestInterval;
    constructor(wc: WebChannel);
    readonly onState: Observable<TopologyStateEnum>;
    readonly state: TopologyStateEnum;
    setStable(): void;
    addJoining(ch: Channel): void;
    initJoining(ch: Channel, ids: number[]): void;
    send(msg: IMessage): void;
    sendTo(msg: IMessage): void;
    forward(msg: IMessage): void;
    leave(): void;
    onChannelClose(event: Event, channel: Channel): void;
    onChannelError(evt: Event): void;
    private clean();
    private handleSvcMsg({channel, senderId, msg});
    private addAdjacentMember(ch);
    private deleteAdjacentMember(ch, quietly?);
    private connectToMany(ids, adjacentId);
    private connectTo(id);
    private membersRequest();
    private sendToDistantPeer(distantPeer, msg);
    private findRoutedChannel(distantPeer, distance);
    private notifyDistantPeers();
    private setState(state);
    private startIntervals();
}
