var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
import { Subject } from 'rxjs/Subject';
import { log } from '../../misc/Util';
import { fullMesh } from '../../proto';
import { SignalingState } from '../../Signaling';
import { Service } from '../Service';
import { WebChannelState } from '../WebChannel';
import { TopologyStateEnum } from './Topology';
/**
 * {@link FullMesh} identifier.
 * @ignore
 * @type {number}
 */
export var FULL_MESH = 3;
var REQUEST_MEMBERS_INTERVAL = 6000;
var MAX_ROUTE_DISTANCE = 3;
var HEARTBEAT_INTERVAL = 3000;
var MAXIMUM_MISSED_HEARTBEAT = 3;
/**
 * Fully connected web channel manager. Implements fully connected topology
 * network, when each peer is connected to each other.
 *
 */
var FullMesh = /** @class */ (function (_super) {
    __extends(FullMesh, _super);
    function FullMesh(wc) {
        var _this = _super.call(this, FULL_MESH, fullMesh.Message, wc.serviceMessageSubject) || this;
        _this.wc = wc;
        _this.adjacentMembers = new Map();
        _this.distantMembers = new Map();
        _this.connectingMembers = new Set();
        _this.stateSubject = new Subject();
        _this.heartbeat = Service.encodeServiceMessage(FULL_MESH, fullMesh.Message.encode(fullMesh.Message.create({ heartbeat: true })).finish());
        _this._state = TopologyStateEnum.DISCONNECTED;
        _this.onServiceMessage.subscribe(function (msg) { return _this.handleSvcMsg(msg); });
        _this.wc.channelBuilder.onChannel.subscribe(function (ch) { return _this.addAdjacentMember(ch); });
        var globalAny = global;
        globalAny.fullmesh = function () {
            log.topology('Fullmesh info:', {
                myId: _this.wc.myId,
                signalingState: SignalingState[_this.wc.signaling.state],
                webGroupState: WebChannelState[_this.wc.state],
                topologyState: TopologyStateEnum[_this._state],
                adjacentMembers: Array.from(_this.adjacentMembers.keys()).toString(),
                distantMembers: Array.from(_this.distantMembers.keys()).toString(),
                connectingMembers: Array.from(_this.connectingMembers.values()).toString(),
                distantMembersDETAILS: Array.from(_this.distantMembers.keys()).map(function (id) {
                    var adjacentMembers = _this.distantMembers.get(id);
                    return { id: id, adjacentMembers: adjacentMembers ? adjacentMembers.adjacentIds : [] };
                }),
            });
        };
        return _this;
    }
    Object.defineProperty(FullMesh.prototype, "onState", {
        get: function () {
            return this.stateSubject.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FullMesh.prototype, "state", {
        get: function () {
            return this._state;
        },
        enumerable: true,
        configurable: true
    });
    FullMesh.prototype.setStable = function () {
        this.setState(TopologyStateEnum.STABLE);
    };
    FullMesh.prototype.addJoining = function (ch) {
        // FIXME: test if `ch` already exists among `adjacentMembers`
        this.addAdjacentMember(ch);
    };
    FullMesh.prototype.initJoining = function (ch, ids) {
        var _this = this;
        this.setState(TopologyStateEnum.JOINING);
        // FIXME: test if `ch` already exists among `adjacentMembers`
        this.addAdjacentMember(ch);
        if (ids.length === 1) {
            this.setState(TopologyStateEnum.STABLE);
        }
        else {
            this.connectToMany(ids, ch.id)
                .then(function () {
                _this.membersRequest();
                _this.setState(TopologyStateEnum.JOINED);
                _this.setState(TopologyStateEnum.STABLE);
            });
        }
    };
    FullMesh.prototype.send = function (msg) {
        var _this = this;
        this.adjacentMembers.forEach(function (ch) { return ch.send(_this.wc.encode(msg)); });
        this.distantMembers.forEach(function (distantPeer, id) {
            _this.sendToDistantPeer(distantPeer, { recipientId: id, isService: msg.isService, content: msg.content });
        });
    };
    FullMesh.prototype.sendTo = function (msg) {
        var ch = this.adjacentMembers.get(msg.recipientId);
        if (ch) {
            ch.send(this.wc.encode(msg));
        }
        else {
            this.sendToDistantPeer(this.distantMembers.get(msg.recipientId), msg);
        }
    };
    FullMesh.prototype.forward = function (msg) {
        if (msg.recipientId > 1) {
            this.sendTo(msg);
        }
    };
    FullMesh.prototype.leave = function () {
        var _this = this;
        if (this._state !== TopologyStateEnum.DISCONNECTED) {
            if (this.adjacentMembers.size === 0) {
                this.clean();
                this.setState(TopologyStateEnum.DISCONNECTED);
            }
            else {
                this.adjacentMembers.forEach(function (ch) { return _this.deleteAdjacentMember(ch); });
            }
        }
    };
    FullMesh.prototype.onChannelClose = function (event, channel) {
        this.deleteAdjacentMember(channel, false);
    };
    FullMesh.prototype.onChannelError = function (evt) {
        log.topology("Channel error: " + evt.type);
    };
    FullMesh.prototype.clean = function () {
        var _this = this;
        this.distantMembers.forEach(function (member, id) { return _this.wc.onMemberLeaveProxy(id); });
        this.distantMembers.clear();
        this.connectingMembers.clear();
        clearInterval(this.heartbeatInterval);
        this.heartbeatInterval = undefined;
        clearInterval(this.membersRequestInterval);
        this.membersRequestInterval = undefined;
    };
    FullMesh.prototype.handleSvcMsg = function (_a) {
        var channel = _a.channel, senderId = _a.senderId, msg = _a.msg;
        switch (msg.type) {
            case 'membersResponse': {
                this.connectToMany(msg.membersResponse.ids, senderId);
                break;
            }
            case 'membersRequest': {
                channel.send(this.wc.encode({
                    recipientId: channel.id,
                    content: _super.prototype.encode.call(this, { membersResponse: { ids: this.wc.members } }),
                }));
                break;
            }
            case 'adjacentMembers': {
                if (!this.adjacentMembers.has(senderId) && senderId !== this.wc.myId) {
                    var distantPeer = this.distantMembers.get(senderId);
                    // If it the first time you have met this ID, then send him your neighbours' ids.
                    // Otherwise just update the list of distant peer's neighbours
                    if (distantPeer) {
                        distantPeer.adjacentIds = msg.adjacentMembers.ids;
                    }
                    else {
                        this.distantMembers.set(senderId, { adjacentIds: msg.adjacentMembers.ids, missedHeartbeat: 0 });
                        var adjacentMembers = _super.prototype.encode.call(this, { adjacentMembers: { ids: Array.from(this.adjacentMembers.keys()) } });
                        this.wc.sendToProxy({ recipientId: senderId, content: adjacentMembers });
                        this.connectTo(senderId);
                    }
                }
                break;
            }
            case 'heartbeat': {
                var distantPeer = this.distantMembers.get(senderId);
                if (distantPeer) {
                    distantPeer.missedHeartbeat = 0;
                }
                else if (channel.id === senderId) {
                    channel.missedHeartbeat = 0;
                }
                break;
            }
        }
    };
    FullMesh.prototype.addAdjacentMember = function (ch) {
        if (this._state === TopologyStateEnum.DISCONNECTED) {
            log.topology('Closing channel quietly as the topology state is: ', TopologyStateEnum[this._state]);
            ch.closeQuietly();
        }
        else {
            log.topology('Adding new adjacent member: ', ch.id);
            this.distantMembers.delete(ch.id);
            var member = this.adjacentMembers.get(ch.id);
            if (member) {
                log.topology('Replacing the same channel');
                member.closeQuietly();
            }
            this.adjacentMembers.set(ch.id, ch);
            this.notifyDistantPeers();
            this.wc.onMemberJoinProxy(ch.id);
            if (this.adjacentMembers.size === 1 && this.membersRequestInterval === undefined) {
                this.startIntervals();
            }
        }
    };
    FullMesh.prototype.deleteAdjacentMember = function (ch, quietly) {
        if (quietly === void 0) { quietly = true; }
        log.topology('Removing adjacent member: ', ch.id);
        this.adjacentMembers.delete(ch.id);
        this.wc.onMemberLeaveProxy(ch.id);
        if (this.adjacentMembers.size === 0) {
            this.clean();
            this.setState(TopologyStateEnum.DISCONNECTED);
        }
        else {
            this.notifyDistantPeers();
        }
        if (quietly) {
            ch.closeQuietly();
        }
    };
    FullMesh.prototype.connectToMany = function (ids, adjacentId) {
        var _this = this;
        if (this._state !== TopologyStateEnum.DISCONNECTED) {
            var missingIds = ids.filter(function (id) {
                return !_this.adjacentMembers.has(id) && !_this.connectingMembers.has(id) && id !== _this.wc.myId;
            });
            if (missingIds.length !== 0) {
                var adjacentMembers_1 = _super.prototype.encode.call(this, { adjacentMembers: { ids: Array.from(this.adjacentMembers.keys()) } });
                var connectingAttempts_1 = [];
                missingIds.forEach(function (id) {
                    if (!_this.distantMembers.has(id)) {
                        _this.distantMembers.set(id, { adjacentIds: [adjacentId], missedHeartbeat: 0 });
                    }
                    // It's important to notify all peers about my adjacentIds first
                    // for some specific case such when you should connect to a peer with
                    // distance more than 1
                    _this.wc.sendToProxy({ recipientId: id, content: adjacentMembers_1 });
                    // Connect only to the peers whose ids are less than my id. Thus it avoids a
                    // problematic situation when both peers are trying to connect to each other
                    // at the same time.
                    connectingAttempts_1[connectingAttempts_1.length] = _this.connectTo(id);
                });
                return Promise.all(connectingAttempts_1);
            }
        }
        return Promise.resolve();
    };
    FullMesh.prototype.connectTo = function (id) {
        var _this = this;
        this.connectingMembers.add(id);
        var result;
        if (id < this.wc.myId) {
            result = this.wc.channelBuilder.connectTo(id)
                .then(function (ch) { return _this.addAdjacentMember(ch); })
                .catch(function (err) {
                if (err.message !== 'pingpong') {
                    return _this.wc.channelBuilder.isResponding(id)
                        .then(function () { return _this.wc.onMemberJoinProxy(id); });
                }
                log.topology(_this.wc.myId + " failed to connect to " + id + ": " + err.message);
                return;
            })
                .catch(function () { });
        }
        else {
            result = this.wc.channelBuilder.isResponding(id)
                .then(function () { return _this.wc.onMemberJoinProxy(id); })
                .catch(function () { });
        }
        return result.then(function () { _this.connectingMembers.delete(id); });
    };
    FullMesh.prototype.membersRequest = function () {
        if (this.adjacentMembers.size !== 0) {
            // Randomly choose a group member to send him a request for his member list
            var index = Math.floor(Math.random() * this.adjacentMembers.size);
            var iterator = this.adjacentMembers.values();
            for (var i = 0; i < index; i++) {
                iterator.next();
            }
            var channel = iterator.next().value;
            channel.send(this.wc.encode({
                recipientId: channel.id,
                content: _super.prototype.encode.call(this, { membersRequest: true }),
            }));
        }
    };
    FullMesh.prototype.sendToDistantPeer = function (distantPeer, msg) {
        // NO LUCK: recepient is not directly connected to me, thus check distant peers
        // First those who are at distance 1, then 2 etc. up to MAX_ROUTE_DISTANCE
        // (Peer X has a distance equals to 1 if I am not directly connected to X and
        // there is a peer Y which is directly connected to X and to me)
        for (var d = 1; d <= MAX_ROUTE_DISTANCE; d++) {
            var ch = this.findRoutedChannel(distantPeer, d);
            if (ch) {
                ch.send(this.wc.encode(msg));
                return;
            }
        }
    };
    FullMesh.prototype.findRoutedChannel = function (distantPeer, distance) {
        if (distantPeer) {
            try {
                for (var _a = __values(this.adjacentMembers), _b = _a.next(); !_b.done; _b = _a.next()) {
                    var _c = __read(_b.value, 2), neighbourId = _c[0], ch = _c[1];
                    if (distantPeer.adjacentIds.includes(neighbourId)) {
                        return ch;
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_b && !_b.done && (_d = _a.return)) _d.call(_a);
                }
                finally { if (e_1) throw e_1.error; }
            }
            if (distance !== 0) {
                try {
                    for (var _e = __values(distantPeer.adjacentIds), _f = _e.next(); !_f.done; _f = _e.next()) {
                        var id = _f.value;
                        var ch = this.findRoutedChannel(this.distantMembers.get(id), distance - 1);
                        if (ch) {
                            return ch;
                        }
                    }
                }
                catch (e_2_1) { e_2 = { error: e_2_1 }; }
                finally {
                    try {
                        if (_f && !_f.done && (_g = _e.return)) _g.call(_e);
                    }
                    finally { if (e_2) throw e_2.error; }
                }
            }
        }
        return undefined;
        var e_1, _d, e_2, _g;
    };
    FullMesh.prototype.notifyDistantPeers = function () {
        var _this = this;
        if (this._state !== TopologyStateEnum.DISCONNECTED) {
            this.distantMembers.forEach(function (peer, id) { return _this.wc.sendToProxy({
                recipientId: id,
                content: _super.prototype.encode.call(_this, { adjacentMembers: { ids: Array.from(_this.adjacentMembers.keys()) } }),
            }); });
        }
    };
    FullMesh.prototype.setState = function (state) {
        if (this._state !== state) {
            this._state = state;
            this.stateSubject.next(state);
        }
    };
    FullMesh.prototype.startIntervals = function () {
        var _this = this;
        // Members check interval
        this.membersRequestInterval = global.setInterval(function () { return _this.membersRequest(); }, REQUEST_MEMBERS_INTERVAL);
        // Heartbeat interval
        this.heartbeatInterval = global.setInterval(function () {
            _this.adjacentMembers.forEach(function (ch) {
                try {
                    ch.missedHeartbeat++;
                    if (ch.missedHeartbeat >= ch.maximumMissedHeartBeat) {
                        throw new Error('Too many missed heartbeats');
                    }
                    ch.sendHeartbeat();
                }
                catch (err) {
                    log.topology("Closing connection with " + ch.id + ". Reason: " + err.message);
                    _this.deleteAdjacentMember(ch);
                }
            });
            _this.distantMembers.forEach(function (peer, id) {
                try {
                    peer.missedHeartbeat++;
                    if (peer.missedHeartbeat >= MAXIMUM_MISSED_HEARTBEAT + 1) {
                        throw new Error('Too many missed heartbeats');
                    }
                    _this.wc.sendToProxy({ recipientId: id, content: _this.heartbeat });
                }
                catch (err) {
                    log.topology("Distant peer " + id + " has left. Reason: " + err.message);
                    _this.distantMembers.delete(id);
                    _this.wc.onMemberLeaveProxy(id);
                }
            });
        }, HEARTBEAT_INTERVAL);
    };
    return FullMesh;
}(Service));
export { FullMesh };
